@@ -5,45 +5,27 @@ from django.forms import (
     CheckboxInput,
     CheckboxSelectMultiple,
     ClearableFileInput,
-    DateInput,
-    EmailInput,
-    FileInput,
     MultiWidget,
-    NumberInput,
-    PasswordInput,
     RadioSelect,
     Select,
-    Textarea,
-    TextInput,
-    TimeInput,
-    URLInput,
 )
+from django.forms.widgets import FileInput, Input, Textarea
 from django.utils.html import conditional_escape, format_html, strip_tags
 from django.utils.safestring import mark_safe
 
 from .core import get_bootstrap_setting
 from .css import merge_css_classes
 from .exceptions import BootstrapError
-from .forms import WRAPPER_CLASS, WRAPPER_TAG, is_widget_with_placeholder, render_field, render_form, render_label
+from .forms import WRAPPER_CLASS, WRAPPER_TAG, render_field, render_form, render_label
+from .size import DEFAULT_SIZE, SIZE_MD, SIZE_XS, get_size_class, parse_size
 from .text import text_value
 from .utils import render_template_file
-
-try:
-    # If Django is set up without a database, importing this widget gives RuntimeError
-    from django.contrib.auth.forms import ReadOnlyPasswordHashWidget
-except RuntimeError:
-    ReadOnlyPasswordHashWidget = None
+from .widgets import ReadOnlyPasswordHashWidget, is_widget_with_placeholder
 
 
 class BaseRenderer(object):
     """A content renderer."""
 
-    SIZE_SM = "sm"
-    SIZE_MD = "md"
-    SIZE_LG = "lg"
-    DEFAULT_SIZE = SIZE_MD
-    SIZES = [SIZE_SM, SIZE_MD, SIZE_LG]
-
     def __init__(self, *args, **kwargs):
         self.layout = kwargs.get("layout", "")
         self.wrapper_class = kwargs.get("wrapper_class", WRAPPER_CLASS)
@@ -52,40 +34,82 @@ class BaseRenderer(object):
         self.show_help = kwargs.get("show_help", True)
         self.show_label = kwargs.get("show_label", True)
         self.exclude = kwargs.get("exclude", "")
-
         self.set_placeholder = kwargs.get("set_placeholder", True)
-        self.size = self.parse_size(kwargs.get("size", ""))
+        self.size = parse_size(kwargs.get("size", ""), default=SIZE_MD)
         self.horizontal_label_class = kwargs.get(
             "horizontal_label_class", get_bootstrap_setting("horizontal_label_class")
         )
         self.horizontal_field_class = kwargs.get(
             "horizontal_field_class", get_bootstrap_setting("horizontal_field_class")
         )
+        self.checkbox_layout = kwargs.get("checkbox_layout", get_bootstrap_setting("checkbox_layout"))
+        self.checkbox_style = kwargs.get("checkbox_style", get_bootstrap_setting("checkbox_style"))
+        self.horizontal_field_offset_class = kwargs.get(
+            "horizontal_field_offset_class", get_bootstrap_setting("horizontal_field_offset_class")
+        )
+        self.inline_field_class = kwargs.get("inline_field_class", get_bootstrap_setting("inline_field_class"))
+        self.error_css_class = kwargs.get("error_css_class", None)
+        self.required_css_class = kwargs.get("required_css_class", None)
+        self.bound_css_class = kwargs.get("bound_css_class", None)
+        self.alert_error_type = kwargs.get("alert_error_type", "non_fields")
 
     @property
     def is_floating(self):
         """Return whether to render `form-control` widgets as floating."""
         return self.layout == "floating"
 
+    @property
+    def is_horizontal(self):
+        """Return whether to render form horizontally."""
+        return self.layout == "horizontal"
+
+    @property
+    def is_inline(self):
+        """Return whether to render widgets with inline layout."""
+        return self.layout == "inline"
+
     def parse_size(self, size):
         """Return size if it is valid, default size if size is empty, or throws exception."""
-        size = text_value(size) or self.DEFAULT_SIZE
-        if size not in self.SIZES:
-            valid_sizes = ", ".join(self.SIZES)
-            raise BootstrapError(f'Invalid value "{size}" for parameter "size" (valid values are {valid_sizes}).')
+        size = parse_size(size, default=DEFAULT_SIZE)
+        if size == SIZE_XS:
+            raise BootstrapError('Size "xs" is not valid for form controls.')
         return size
 
     def get_size_class(self, prefix):
         """Return size class for given prefix."""
-        return f"{prefix}-{self.size}" if self.size in ["sm", "lg"] else ""
+        return get_size_class(self.size, prefix=prefix) if self.size in ["sm", "lg"] else ""
 
-    def _render(self):
-        """Render to string."""
-        return ""
+    def get_kwargs(self):
+        """Return kwargs to pass on to child renderers."""
+        context = {
+            "layout": self.layout,
+            "wrapper_class": self.wrapper_class,
+            "field_class": self.field_class,
+            "label_class": self.label_class,
+            "show_help": self.show_help,
+            "show_label": self.show_label,
+            "exclude": self.exclude,
+            "set_placeholder": self.set_placeholder,
+            "size": self.size,
+            "horizontal_label_class": self.horizontal_label_class,
+            "horizontal_field_class": self.horizontal_field_class,
+            "checkbox_layout": self.checkbox_layout,
+            "checkbox_style": self.checkbox_style,
+            "inline_field_class": self.inline_field_class,
+            "error_css_class": self.error_css_class,
+            "bound_css_class": self.bound_css_class,
+            "required_css_class": self.required_css_class,
+            "alert_error_type": self.alert_error_type,
+        }
+        return context
+
+    def get_context_data(self):
+        """Return context data for rendering."""
+        return self.get_kwargs()
 
     def render(self):
-        """Render to safe string."""
-        return mark_safe(self._render())
+        """Render to string."""
+        return ""
 
 
 class FormsetRenderer(BaseRenderer):
@@ -97,30 +121,21 @@ class FormsetRenderer(BaseRenderer):
         self.formset = formset
         super().__init__(*args, **kwargs)
 
+    def get_context_data(self):
+        context = super().get_context_data()
+        context["formset"] = self.formset
+        return context
+
     def render_management_form(self):
         """Return HTML for management form."""
         return text_value(self.formset.management_form)
 
     def render_forms(self):
-        rendered_forms = []
+        rendered_forms = mark_safe("")
+        kwargs = self.get_kwargs()
         for form in self.formset.forms:
-            rendered_forms.append(
-                render_form(
-                    form,
-                    layout=self.layout,
-                    form_group_class=self.wrapper_class,
-                    field_class=self.field_class,
-                    label_class=self.label_class,
-                    show_label=self.show_label,
-                    show_help=self.show_help,
-                    exclude=self.exclude,
-                    set_placeholder=self.set_placeholder,
-                    size=self.size,
-                    horizontal_label_class=self.horizontal_label_class,
-                    horizontal_field_class=self.horizontal_field_class,
-                )
-            )
-        return "\n".join(rendered_forms)
+            rendered_forms += render_form(form, **kwargs)
+        return rendered_forms
 
     def get_formset_errors(self):
         return self.formset.non_form_errors()
@@ -130,12 +145,16 @@ class FormsetRenderer(BaseRenderer):
         if formset_errors:
             return render_template_file(
                 "django_bootstrap5/form_errors.html",
-                context={"errors": formset_errors, "form": self.formset, "layout": self.layout},
+                context={
+                    "errors": formset_errors,
+                    "form": self.formset,
+                    "layout": self.layout,
+                },
             )
-        return ""
+        return mark_safe("")
 
-    def _render(self):
-        return "".join([self.render_errors(), self.render_management_form(), self.render_forms()])
+    def render(self):
+        return format_html(self.render_management_form() + "{}{}", self.render_errors(), self.render_forms())
 
 
 class FormRenderer(BaseRenderer):
@@ -146,36 +165,18 @@ class FormRenderer(BaseRenderer):
             raise BootstrapError('Parameter "form" should contain a valid Django Form.')
         self.form = form
         super().__init__(*args, **kwargs)
-        self.error_css_class = kwargs.get("error_css_class", None)
-        self.required_css_class = kwargs.get("required_css_class", None)
-        self.bound_css_class = kwargs.get("bound_css_class", None)
-        self.alert_error_type = kwargs.get("alert_error_type", "non_fields")
-        self.form_check_class = kwargs.get("form_check_class", "form-check")
+
+    def get_context_data(self):
+        context = super().get_context_data()
+        context["form"] = self.form
+        return context
 
     def render_fields(self):
-        rendered_fields = []
+        rendered_fields = mark_safe("")
+        kwargs = self.get_kwargs()
         for field in self.form:
-            rendered_fields.append(
-                render_field(
-                    field,
-                    layout=self.layout,
-                    form_group_class=self.wrapper_class,
-                    field_class=self.field_class,
-                    label_class=self.label_class,
-                    form_check_class=self.form_check_class,
-                    show_label=self.show_label,
-                    show_help=self.show_help,
-                    exclude=self.exclude,
-                    set_placeholder=self.set_placeholder,
-                    size=self.size,
-                    horizontal_label_class=self.horizontal_label_class,
-                    horizontal_field_class=self.horizontal_field_class,
-                    error_css_class=self.error_css_class,
-                    required_css_class=self.required_css_class,
-                    bound_css_class=self.bound_css_class,
-                )
-            )
-        return "\n".join(rendered_fields)
+            rendered_fields += render_field(field, **kwargs)
+        return rendered_fields
 
     def get_fields_errors(self):
         form_errors = []
@@ -199,28 +200,17 @@ class FormRenderer(BaseRenderer):
                 context={"errors": form_errors, "form": self.form, "layout": self.layout, "type": type},
             )
 
-        return ""
+        return mark_safe("")
 
-    def _render(self):
-        return self.render_errors(self.alert_error_type) + self.render_fields()
+    def render(self):
+        errors = self.render_errors(self.alert_error_type)
+        fields = self.render_fields()
+        return errors + fields
 
 
 class FieldRenderer(BaseRenderer):
     """Default field renderer."""
 
-    FORM_CONTROL_WIDGETS = (
-        TextInput,
-        NumberInput,
-        EmailInput,
-        URLInput,
-        DateInput,
-        TimeInput,
-        Textarea,
-        PasswordInput,
-        FileInput,
-    )
-    FLOATING_WIDGETS = (TextInput, NumberInput, EmailInput, URLInput, DateInput, TimeInput, Textarea, PasswordInput)
-
     def __init__(self, field, *args, **kwargs):
         if not isinstance(field, BoundField):
             raise BootstrapError('Parameter "field" should contain a valid Django BoundField.')
@@ -232,19 +222,8 @@ class FieldRenderer(BaseRenderer):
         self.initial_attrs = self.widget.attrs.copy()
         self.help_text = text_value(field.help_text) if self.show_help and field.help_text else ""
         self.field_errors = [conditional_escape(text_value(error)) for error in field.errors]
-        self.form_check_class = kwargs.get("form_check_class", "form-check")
 
-        if "placeholder" in kwargs:
-            # Find the placeholder in kwargs, even if it's empty
-            self.placeholder = kwargs["placeholder"]
-        elif get_bootstrap_setting("set_placeholder"):
-            # If not found, see if we set the label
-            self.placeholder = field.label
-        else:
-            # Or just set it to empty
-            self.placeholder = ""
-        if self.placeholder:
-            self.placeholder = text_value(self.placeholder)
+        self.placeholder = text_value(kwargs.get("placeholder", self.default_placeholder))
 
         self.addon_before = kwargs.get("addon_before", self.widget.attrs.pop("addon_before", ""))
         self.addon_after = kwargs.get("addon_after", self.widget.attrs.pop("addon_after", ""))
@@ -283,44 +262,65 @@ class FieldRenderer(BaseRenderer):
     def is_floating(self):
         return super().is_floating and self.can_widget_float(self.widget) and self.is_widget_form_control(self.widget)
 
+    @property
+    def default_placeholder(self):
+        """Return default placeholder for field."""
+        return self.field.label if get_bootstrap_setting("set_placeholder") else ""
+
     def restore_widget_attrs(self):
         self.widget.attrs = self.initial_attrs.copy()
 
     def is_widget_form_control(self, widget):
         """Return whether given widget is of type `form-control`."""
-        return isinstance(widget, self.FORM_CONTROL_WIDGETS)
+        return (isinstance(widget, Input) and not isinstance(widget, CheckboxInput)) or isinstance(widget, Textarea)
+
+    def get_widget_input_type(self, widget):
+        """Return input type of widget, or None."""
+        return widget.input_type if isinstance(widget, Input) else None
 
     def can_widget_float(self, widget):
         """Return whether given widget can be set to `form-floating` behavior."""
-        return isinstance(widget, self.FLOATING_WIDGETS)
+        # TODO: Add support for select widgets, within Bootstrap 5 restrictions
+        # TODO: Add support for textarea widgets
+        # TODO: Check support for date, time and other types
+        return (
+            not isinstance(widget, FileInput)
+            and self.is_widget_form_control(widget)
+            and self.get_widget_input_type(widget) != "color"
+        )
 
     def add_widget_class_attrs(self, widget=None):
         """Add class attribute to widget."""
         if widget is None:
             widget = self.widget
         size_prefix = None
-        classes = widget.attrs.get("class", "")
 
+        before = []
+        classes = [widget.attrs.get("class", "")]
         if ReadOnlyPasswordHashWidget is not None and isinstance(widget, ReadOnlyPasswordHashWidget):
-            classes = merge_css_classes("form-control-static", classes)
+            before.append("form-control-static")
         elif self.is_widget_form_control(widget):
-            classes = merge_css_classes("form-control", classes)
+            before.append("form-control")
+            if self.get_widget_input_type(widget) == "color":
+                before.append("form-control-color")
             size_prefix = "form-control"
         elif isinstance(widget, Select):
-            classes = merge_css_classes("form-select", classes)
+            before.append("form-select")
             size_prefix = "form-select"
         elif isinstance(widget, CheckboxInput):
-            classes = merge_css_classes("form-check-input", classes)
+            before.append("form-check-input")
+
         if size_prefix:
-            classes = merge_css_classes(classes, self.get_size_class(prefix=size_prefix))
+            classes.append(get_size_class(self.size, prefix=size_prefix, skip=["xs", "md"]))
 
         if self.field.errors:
             if self.error_css_class:
-                classes = merge_css_classes(classes, self.error_css_class)
+                classes.append(self.error_css_class)
         elif self.field.form.is_bound:
-            classes = merge_css_classes(classes, self.success_css_class)
+            classes.append(self.success_css_class)
 
-        widget.attrs["class"] = classes
+        classes = before + classes
+        widget.attrs["class"] = merge_css_classes(*classes)
 
     def add_placeholder_attrs(self, widget=None):
         """Add placeholder attribute to widget."""
@@ -344,7 +344,7 @@ class FieldRenderer(BaseRenderer):
             elif isinstance(widget, ClearableFileInput):
                 widget.template_name = "django_bootstrap5/widgets/clearable_file_input.html"
 
-    def get_label_class(self):
+    def get_label_class(self, horizontal=False):
         """Return CSS class for label."""
         label_classes = [text_value(self.label_class)]
         if not self.show_label:
@@ -352,6 +352,10 @@ class FieldRenderer(BaseRenderer):
         else:
             if isinstance(self.widget, CheckboxInput):
                 widget_label_class = "form-check-label"
+            elif self.is_inline:
+                widget_label_class = "visually-hidden"
+            elif horizontal:
+                widget_label_class = merge_css_classes(self.horizontal_label_class, "col-form-label")
             else:
                 widget_label_class = "form-label"
             label_classes = [widget_label_class] + label_classes
@@ -364,11 +368,15 @@ class FieldRenderer(BaseRenderer):
         self.restore_widget_attrs()
         return field_html
 
-    def get_label_html(self):
+    def get_label_html(self, horizontal=False):
         """Return value for label."""
         label_html = "" if self.show_label == "skip" else self.field.label
         if label_html:
-            label_html = render_label(label_html, label_for=self.field.id_for_label, label_class=self.get_label_class())
+            label_html = render_label(
+                label_html,
+                label_for=self.field.id_for_label,
+                label_class=self.get_label_class(horizontal=horizontal),
+            )
         return label_html
 
     def get_help_html(self):
@@ -401,6 +409,19 @@ class FieldRenderer(BaseRenderer):
             )
         return ""
 
+    def get_inline_field_class(self):
+        """Return CSS class for inline field."""
+        return self.inline_field_class or "col-12"
+
+    def get_checkbox_classes(self):
+        """Return CSS classes for checkbox."""
+        classes = ["form-check"]
+        if self.checkbox_style == "switch":
+            classes.append("form-switch")
+        if self.checkbox_layout == "inline":
+            classes.append("form-check-inline")
+        return merge_css_classes(*classes)
+
     def get_wrapper_classes(self):
         """Return classes for wrapper."""
         wrapper_classes = [self.wrapper_class]
@@ -412,33 +433,51 @@ class FieldRenderer(BaseRenderer):
             wrapper_classes.append(self.success_css_class)
         if self.field.field.required:
             wrapper_classes.append(self.required_css_class)
-        wrapper_classes.append("mb-3")
+        if self.is_inline:
+            wrapper_classes.append(self.get_inline_field_class())
+        else:
+            if self.is_horizontal:
+                wrapper_classes.append("row")
+            wrapper_classes.append("mb-3")
         return merge_css_classes(*wrapper_classes)
 
     def field_before_label(self):
         """Return whether field should be placed before label."""
         return isinstance(self.widget, CheckboxInput) or self.is_floating
 
-    def _render(self):
+    def render(self):
         if self.field.name in self.exclude.replace(" ", "").split(","):
             return ""
         if self.field.is_hidden:
             return text_value(self.field)
 
         field = self.get_field_html()
-
-        label = self.get_label_html()
-
-        field_with_label = field + label if self.field_before_label() else label + field
+        if self.field_before_label():
+            label = self.get_label_html()
+            field = field + label
+            label = mark_safe("")
+            horizontal_class = merge_css_classes(self.horizontal_field_class, self.horizontal_field_offset_class)
+        else:
+            label = self.get_label_html(horizontal=self.is_horizontal)
+            horizontal_class = self.horizontal_field_class
 
         if isinstance(self.widget, CheckboxInput):
-            field_with_label = format_html('<div class="form-check">{}</div>', field_with_label)
+            field = format_html(
+                '<div class="{form_check_class}">{field}</div>',
+                form_check_class=self.get_checkbox_classes(),
+                field=field,
+            )
+
+        field_with_help_and_errors = format_html("{}{}{}", field, self.get_help_html(), self.get_errors_html())
+        if self.is_horizontal:
+            field_with_help_and_errors = format_html(
+                '<div class="{}">{}</div>', horizontal_class, field_with_help_and_errors
+            )
 
         return format_html(
-            '<{tag} class="{wrapper_classes}">{field_with_label}{help}{errors}</{tag}>',
+            '<{tag} class="{wrapper_classes}">{label}{field_with_help_and_errors}</{tag}>',
             tag=WRAPPER_TAG,
             wrapper_classes=self.get_wrapper_classes(),
-            field_with_label=field_with_label,
-            help=self.get_help_html(),
-            errors=self.get_errors_html(),
+            label=label,
+            field_with_help_and_errors=field_with_help_and_errors,
         )
